/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.job;

import org.json.simple.JSONObject;
import org.quartz.CronExpression;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.GroupMatcher;

import static org.quartz.JobBuilder.*;
import static org.quartz.CronScheduleBuilder.*;
import static org.quartz.TriggerBuilder.*;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.log.Log;
import re.jpayet.mentdb.core.script.ScriptManager;
import re.jpayet.mentdb.core.session.Session;

//The job class
public class JobManager implements Job {
	
	//The constructor
	public JobManager() {
		
	}

	//Job execution
	public void execute(JobExecutionContext context) throws JobExecutionException {
		
		//initialization
	    JobDataMap dataMap = context.getJobDetail().getJobDataMap();
	    String scriptName = dataMap.getString("scriptName");
	    
	    //Try to execute the command
		try {
			
			//MentDB mode
			Session.allConnections.get("ai").execute("execute "+scriptName);
			
		} catch (Exception e) {
			
			//Write the error message
			Log.write("JOB ERROR: "+e.getMessage());
			
		}
		
	}
	
	//Initialization
	public static Scheduler scheduler = null;
	
	//Initialize the job manager
	public static void init(String user) throws Exception {

		JSONObject dataNode = new JSONObject();
		
		Record.add(user, "JOB[]", dataNode.toJSONString());

	}

	//Add a job
	@SuppressWarnings("unchecked")
	public static void add(String user, String jobId, String scriptName, String pattern) throws Exception {
		
		//Generate an error if the job already exist
		if (exist(jobId)) {

			throw new Exception("Sorry, the job "+jobId+" already exist.");

		}
		
		//Generate an error if the script does not exist
		if (!ScriptManager.exist(scriptName)) {

			throw new Exception("Sorry, the script "+scriptName+" does not exist.");

		}
		
		//Check the pattern
		try {
			
			new CronExpression(pattern);
			
		} catch (Exception e) {
			
			throw new Exception("Sorry, the job pattern "+pattern+" is not valid.");
			
		}
		
		//Get the JOB object
		JSONObject bd = Record.getNode("JOB[]");
		
		JSONObject job = new JSONObject();
		job.put("id", jobId);
		job.put("scriptName", scriptName);
		job.put("pattern", pattern);
		
		bd.put(jobId, job);

		Record.update(user, "JOB[]", bd.toJSONString());
		
	}

	//Check if a job already exist
	public static boolean exist(String jobId) throws Exception {
		
		//Get the JOB object
		JSONObject bd = Record.getNode("JOB[]");
		
		return bd.containsKey(jobId);
		
	}

	//Show all jobs
	public static JSONObject show() throws Exception {
		
		return Record.getNode("JOB[]");
		
	}

	//Generate update
	public static String generateUpdate(String jobId) throws Exception {
		
		//Generate an error if the job does not exist
		if (!exist(jobId)) {

			throw new Exception("Sorry, the job "+jobId+" does not exist.");

		}
		
		//Get the JOB object
		JSONObject bd = Record.getNode("JOB[]");
		
		JSONObject j = ((JSONObject) bd.get(jobId));
		
		String result = "job update \""+jobId+"\" \""+j.get("scriptName")+"\" \""+j.get("pattern")+"\";";
		
		return result;
		
	}

	//Stop all jobs
	public static void scheduler_stop() throws Exception {
		
		//Generate an error if the job scheduler is already stopped
		if (scheduler == null) {

			throw new Exception("Sorry, the job scheduler is already stopped.");

		}
		
		//Delete all jobs from the scheduler
		for(String group: scheduler.getJobGroupNames()) {
		   for(JobKey jobKey : scheduler.getJobKeys(GroupMatcher.jobGroupEquals(group))) {
			   scheduler.deleteJob(jobKey);
		   }
		}
		
		scheduler.shutdown();
		scheduler = null;
		
	}

	//Start all jobs
	public static void scheduler_start() throws Exception {
		
		//Generate an error if the job scheduler is already started
		if (scheduler!=null) {

			throw new Exception("Sorry, the job scheduler is already started.");

		}
		
		scheduler = StdSchedulerFactory.getDefaultScheduler();
		
		JSONObject jobs = show();
		
		//parse all jobs
		for (Object entry : jobs.keySet()) {
			
			String key = entry.toString();

			String pattern = ((JSONObject) jobs.get(key)).get("pattern")+"";
			String scriptName = ((JSONObject) jobs.get(key)).get("scriptName")+"";
			
			//Create the job
			JobDetail job = newJob(JobManager.class)
				      .withIdentity("J"+key, "Jobs")
				      .usingJobData("scriptName", scriptName)
				      .build();
		     
			//Create the trigger
	         Trigger trigger = newTrigger()
	        		    .withIdentity("T"+key, "Jobs")
	        		    .startNow()
	        		    .withSchedule(cronSchedule(pattern))
	        		    .build();
		    
	        //Schedule the job
		    scheduler.scheduleJob(job, trigger);
			
	    }
		
		scheduler.start();
		
	}

	//Reload all jobs
	public static void scheduler_restart() throws Exception {
		
		//Stop all jobs
		scheduler_stop();
		
		//Start all jobs
		scheduler_start();
		
	}

	//Get the status of the scheduler
	public static String scheduler_status() throws Exception {
		
		if (scheduler==null) return "Stopped.";
		else return "Running...";
		
	}

	//Update a stimulation
	@SuppressWarnings("unchecked")
	public static void update(String user, String jobId, String scriptName, String pattern) throws Exception {
		
		//Generate an error if the job does not exist
		if (!exist(jobId)) {

			throw new Exception("Sorry, the job "+jobId+" does not exist.");

		}
		
		//Generate an error if the script does not exist
		if (!ScriptManager.exist(scriptName)) {

			throw new Exception("Sorry, the script "+scriptName+" does not exist.");

		}
		
		//Check the pattern
		try {
			
			new CronExpression(pattern);
			
		} catch (Exception e) {
			
			throw new Exception("Sorry, the job pattern "+pattern+" is not valid.");
			
		}
		
		//Get the JOB object
		JSONObject bd = Record.getNode("JOB[]");
		
		JSONObject job = new JSONObject();
		job.put("id", jobId);
		job.put("scriptName", scriptName);
		job.put("pattern", pattern);
		
		bd.put(jobId, job);

		Record.update(user, "JOB[]", bd.toJSONString());
		
	}

	//Delete a stimulation
	public static void delete(String user, String jobId) throws Exception {
		
		//Generate an error if the job does not exist
		if (!exist(jobId)) {

			throw new Exception("Sorry, the job "+jobId+" does not exist.");

		}
		
		//Get the JOB object
		JSONObject bd = Record.getNode("JOB[]");
		
		bd.remove(jobId);

		Record.update(user, "JOB[]", bd.toJSONString());
		
	}

}
